@@ -28,7 +28,6 @@
 
 #include <glib/gi18n.h>
 
-#define DBUS_API_SUBJECT_TO_CHANGE
 #include <dbus/dbus.h>
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
@@ -65,6 +64,10 @@ static DBusHandlerResult gs_listener_message_handler    (DBusConnection  *connec
 #define CK_MANAGER_INTERFACE "org.freedesktop.ConsoleKit.Manager"
 #define CK_SESSION_INTERFACE "org.freedesktop.ConsoleKit.Session"
 
+#define SESSION_NAME         "org.gnome.SessionManager"
+#define SESSION_PATH         "/org/gnome/SessionManager"
+#define SESSION_INTERFACE    "org.gnome.SessionManager"
+
 #define TYPE_MISMATCH_ERROR GS_LISTENER_INTERFACE ".TypeMismatch"
 
 #define GS_LISTENER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GS_TYPE_LISTENER, GSListenerPrivate))
@@ -94,6 +97,7 @@ typedef struct
         char    *reason;
         char    *connection;
         guint32  cookie;
+        guint32  foreign_cookie;
         GTimeVal since;
 } GSListenerRefEntry;
 
@@ -207,50 +211,6 @@ send_dbus_boolean_signal (GSListener *listener,
         dbus_message_unref (message);
 }
 
-static void
-send_dbus_void_signal (GSListener *listener,
-                       const char *name)
-{
-        DBusMessage  *message;
-
-        g_return_if_fail (listener != NULL);
-
-        message = dbus_message_new_signal (GS_LISTENER_PATH,
-                                           GS_LISTENER_SERVICE,
-                                           name);
-
-        if (! send_dbus_message (listener->priv->connection, message)) {
-                gs_debug ("Could not send %s signal", name);
-        }
-
-        dbus_message_unref (message);
-}
-
-void
-gs_listener_emit_power_notice (GSListener *listener,
-                               gboolean    in_effect)
-{
-        g_return_if_fail (listener != NULL);
-
-        send_dbus_boolean_signal (listener, "SessionPowerManagementIdleChanged", in_effect);
-}
-
-void
-gs_listener_emit_auth_request_begin (GSListener *listener)
-{
-        g_return_if_fail (listener != NULL);
-
-        send_dbus_void_signal (listener, "AuthenticationRequestBegin");
-}
-
-void
-gs_listener_emit_auth_request_end (GSListener *listener)
-{
-        g_return_if_fail (listener != NULL);
-
-        send_dbus_void_signal (listener, "AuthenticationRequestEnd");
-}
-
 static void
 gs_listener_send_signal_active_changed (GSListener *listener)
 {
@@ -262,68 +222,6 @@ gs_listener_send_signal_active_changed (GSListener *listener)
         send_dbus_boolean_signal (listener, "ActiveChanged", listener->priv->active);
 }
 
-static void
-gs_listener_send_signal_session_idle_changed (GSListener *listener)
-{
-        g_return_if_fail (listener != NULL);
-
-        send_dbus_boolean_signal (listener, "SessionIdleChanged", listener->priv->session_idle);
-}
-
-static void
-gs_listener_update_console_kit_idle (GSListener *listener)
-{
-        DBusMessage    *message;
-        DBusMessage    *reply;
-        DBusError       error;
-        DBusMessageIter iter;
-        gboolean        idle;
-
-        g_return_if_fail (listener != NULL);
-
-        if (listener->priv->session_id == NULL) {
-                /* we don't have ConsoleKit running booo! */
-                return;
-        }
-
-        if (listener->priv->system_connection == NULL) {
-                gs_debug ("No connection to the system bus");
-                return;
-        }
-
-        idle = listener->priv->session_idle;
-
-        gs_debug ("Updating ConsoleKit idle status: %d", idle);
-        message = dbus_message_new_method_call (CK_NAME,
-                                                listener->priv->session_id,
-                                                CK_SESSION_INTERFACE,
-                                                "SetIdleHint");
-        if (message == NULL) {
-                gs_debug ("Couldn't allocate the D-Bus message");
-                return;
-        }
-
-        dbus_message_iter_init_append (message, &iter);
-        dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &idle);
-
-        /* FIXME: use async? */
-        dbus_error_init (&error);
-        reply = dbus_connection_send_with_reply_and_block (listener->priv->system_connection,
-                                                           message,
-                                                           -1, &error);
-        dbus_message_unref (message);
-
-        if (reply != NULL) {
-                dbus_message_unref (reply);
-        }
-
-        if (dbus_error_is_set (&error)) {
-                gs_debug ("%s raised:\n %s\n\n", error.name, error.message);
-                dbus_error_free (&error);
-                return;
-        }
-}
-
 static const char *
 get_name_for_entry_type (int entry_type)
 {
@@ -474,10 +372,6 @@ listener_set_session_idle_internal (GSListener *listener,
                 listener->priv->session_idle_start = 0;
         }
 
-        gs_listener_send_signal_session_idle_changed (listener);
-
-        gs_listener_update_console_kit_idle (listener);
-
         return TRUE;
 }
 
@@ -709,6 +603,104 @@ listener_ref_entry_check (GSListener *listener,
         }
 }
 
+static void
+add_session_inhibit (GSListener         *listener,
+                     GSListenerRefEntry *entry)
+{
+        DBusMessage    *message;
+        DBusMessage    *reply;
+        DBusMessageIter iter;
+        DBusMessageIter reply_iter;
+        DBusError       error;
+        guint           xid;
+        guint           flags;
+
+        g_return_if_fail (listener != NULL);
+
+        dbus_error_init (&error);
+
+        message = dbus_message_new_method_call (SESSION_NAME,
+                                                SESSION_PATH,
+                                                SESSION_INTERFACE,
+                                                "Inhibit");
+        if (message == NULL) {
+                gs_debug ("Couldn't allocate the dbus message");
+                return;
+        }
+
+        dbus_message_iter_init_append (message, &iter);
+        xid = 0;
+        flags = 8;
+        dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &entry->application);
+        dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &xid);
+        dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &entry->reason);
+        dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &flags);
+
+        /* FIXME: use async? */
+        reply = dbus_connection_send_with_reply_and_block (listener->priv->connection,
+                                                           message,
+                                                           -1,
+                                                           &error);
+        dbus_message_unref (message);
+
+        if (dbus_error_is_set (&error)) {
+                gs_debug ("%s raised:\n %s\n\n", error.name, error.message);
+                dbus_error_free (&error);
+                return;
+        }
+
+        dbus_message_iter_init (reply, &reply_iter);
+        dbus_message_iter_get_basic (&reply_iter, &entry->foreign_cookie);
+
+        dbus_message_unref (reply);
+}
+
+static void
+remove_session_inhibit (GSListener         *listener,
+                        GSListenerRefEntry *entry)
+{
+        DBusMessage    *message;
+        DBusMessage    *reply;
+        DBusMessageIter iter;
+        DBusError       error;
+
+        g_return_if_fail (listener != NULL);
+
+        if (entry->foreign_cookie == 0) {
+                gs_debug ("Can't remove inhibitor from session: Session cookie not set");
+                return;
+        }
+
+        dbus_error_init (&error);
+
+        message = dbus_message_new_method_call (SESSION_NAME,
+                                                SESSION_PATH,
+                                                SESSION_INTERFACE,
+                                                "Uninhibit");
+        if (message == NULL) {
+                gs_debug ("Couldn't allocate the dbus message");
+                return;
+        }
+
+        dbus_message_iter_init_append (message, &iter);
+        dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &entry->foreign_cookie);
+
+        /* FIXME: use async? */
+        reply = dbus_connection_send_with_reply_and_block (listener->priv->connection,
+                                                           message,
+                                                           -1,
+                                                           &error);
+        dbus_message_unref (message);
+
+        if (dbus_error_is_set (&error)) {
+                gs_debug ("%s raised:\n %s\n\n", error.name, error.message);
+                dbus_error_free (&error);
+                return;
+        }
+
+        dbus_message_unref (reply);
+}
+
 static void
 listener_add_ref_entry (GSListener         *listener,
                         int                 entry_type,
@@ -725,6 +717,11 @@ listener_add_ref_entry (GSListener         *listener,
         hash = get_hash_for_entry_type (listener, entry_type);
         g_hash_table_insert (hash, &entry->cookie, entry);
 
+        if (entry_type == REF_ENTRY_TYPE_INHIBIT) {
+                /* proxy inhibit over to gnome session */
+                add_session_inhibit (listener, entry);
+        }
+
         listener_ref_entry_check (listener, entry_type);
 }
 
@@ -748,6 +745,11 @@ listener_remove_ref_entry (GSListener *listener,
                           entry->connection);
         }
 
+        if (entry_type == REF_ENTRY_TYPE_INHIBIT) {
+                /* remove inhibit from gnome session */
+                remove_session_inhibit (listener, entry);
+        }
+
         removed = g_hash_table_remove (hash, &cookie);
 
         if (removed) {
@@ -1155,13 +1157,6 @@ listener_get_property (GSListener     *listener,
                         dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &b);
                 }
                 break;
-        case PROP_SESSION_IDLE:
-                {
-                        dbus_bool_t b;
-                        b = listener->priv->session_idle;
-                        dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &b);
-                }
-                break;
         default:
                 gs_debug ("Unsupported property id %u", prop_id);
                 break;
@@ -1223,53 +1218,6 @@ listener_get_active_time (GSListener     *listener,
         return DBUS_HANDLER_RESULT_HANDLED;
 }
 
-static DBusHandlerResult
-listener_get_session_idle_time (GSListener     *listener,
-                                DBusConnection *connection,
-                                DBusMessage    *message)
-{
-        DBusMessageIter iter;
-        DBusMessage    *reply;
-        dbus_uint32_t    secs;
-
-        reply = dbus_message_new_method_return (message);
-
-        if (reply == NULL) {
-                g_error ("No memory");
-        }
-
-        dbus_message_iter_init_append (reply, &iter);
-
-        if (listener->priv->session_idle) {
-                time_t now = time (NULL);
-
-                if (now < listener->priv->session_idle_start) {
-                        /* shouldn't happen */
-                        gs_debug ("Session idle start time is in the future");
-                        secs = 0;
-                } else if (listener->priv->session_idle_start <= 0) {
-                        /* shouldn't happen */
-                        gs_debug ("Session idle start time was not set");
-                        secs = 0;
-                } else {
-                        secs = now - listener->priv->session_idle_start;
-                }
-        } else {
-                secs = 0;
-        }
-
-        gs_debug ("Returning session idle for %u seconds", secs);
-        dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &secs);
-
-        if (! dbus_connection_send (connection, reply, NULL)) {
-                g_error ("No memory");
-        }
-
-        dbus_message_unref (reply);
-
-        return DBUS_HANDLER_RESULT_HANDLED;
-}
-
 static DBusHandlerResult
 do_introspect (DBusConnection *connection,
                DBusMessage    *message,
@@ -1326,25 +1274,9 @@ do_introspect (DBusConnection *connection,
                                "    <method name=\"SetActive\">\n"
                                "      <arg name=\"value\" direction=\"in\" type=\"b\"/>\n"
                                "    </method>\n"
-                               "    <method name=\"GetSessionIdle\">\n"
-                               "      <arg name=\"value\" direction=\"out\" type=\"b\"/>\n"
-                               "    </method>\n"
-                               "    <method name=\"GetSessionIdleTime\">\n"
-                               "      <arg name=\"seconds\" direction=\"out\" type=\"u\"/>\n"
-                               "    </method>\n"
                                "    <signal name=\"ActiveChanged\">\n"
                                "      <arg name=\"new_value\" type=\"b\"/>\n"
                                "    </signal>\n"
-                               "    <signal name=\"SessionIdleChanged\">\n"
-                               "      <arg name=\"new_value\" type=\"b\"/>\n"
-                               "    </signal>\n"
-                               "    <signal name=\"SessionPowerManagementIdleChanged\">\n"
-                               "      <arg name=\"new_value\" type=\"b\"/>\n"
-                               "    </signal>\n"
-                               "    <signal name=\"AuthenticationRequestBegin\">\n"
-                               "    </signal>\n"
-                               "    <signal name=\"AuthenticationRequestEnd\">\n"
-                               "    </signal>\n"
                                "  </interface>\n");
 
         reply = dbus_message_new_method_return (message);
@@ -1426,12 +1358,6 @@ listener_dbus_handle_session_message (DBusConnection *connection,
         if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "GetActiveTime")) {
                 return listener_get_active_time (listener, connection, message);
         }
-        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "GetSessionIdle")) {
-                return listener_get_property (listener, connection, message, PROP_SESSION_IDLE);
-        }
-        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "GetSessionIdleTime")) {
-                return listener_get_session_idle_time (listener, connection, message);
-        }
         if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "SimulateUserActivity")) {
                 g_signal_emit (listener, signals [SIMULATE_USER_ACTIVITY], 0);
                 return DBUS_HANDLER_RESULT_HANDLED;
@@ -1973,28 +1899,33 @@ gs_listener_acquire (GSListener *listener,
                             ",member='NameOwnerChanged'",
                             NULL);
 
-        dbus_connection_add_filter (listener->priv->system_connection, listener_dbus_system_filter_function, listener, NULL);
+        if (listener->priv->system_connection != NULL) {
+                dbus_connection_add_filter (listener->priv->system_connection,
+                                            listener_dbus_system_filter_function,
+                                            listener,
+                                            NULL);
 
-        dbus_bus_add_match (listener->priv->system_connection,
-                            "type='signal'"
-                            ",interface='"HAL_DEVICE_INTERFACE"'"
-                            ",member='Condition'",
-                            NULL);
-        dbus_bus_add_match (listener->priv->system_connection,
-                            "type='signal'"
-                            ",interface='"CK_SESSION_INTERFACE"'"
-                            ",member='Unlock'",
-                            NULL);
-        dbus_bus_add_match (listener->priv->system_connection,
-                            "type='signal'"
-                            ",interface='"CK_SESSION_INTERFACE"'"
-                            ",member='Lock'",
-                            NULL);
-        dbus_bus_add_match (listener->priv->system_connection,
-                            "type='signal'"
-                            ",interface='"CK_SESSION_INTERFACE"'"
-                            ",member='ActiveChanged'",
-                            NULL);
+                dbus_bus_add_match (listener->priv->system_connection,
+                                    "type='signal'"
+                                    ",interface='"HAL_DEVICE_INTERFACE"'"
+                                    ",member='Condition'",
+                                    NULL);
+                dbus_bus_add_match (listener->priv->system_connection,
+                                    "type='signal'"
+                                    ",interface='"CK_SESSION_INTERFACE"'"
+                                    ",member='Unlock'",
+                                    NULL);
+                dbus_bus_add_match (listener->priv->system_connection,
+                                    "type='signal'"
+                                    ",interface='"CK_SESSION_INTERFACE"'"
+                                    ",member='Lock'",
+                                    NULL);
+                dbus_bus_add_match (listener->priv->system_connection,
+                                    "type='signal'"
+                                    ",interface='"CK_SESSION_INTERFACE"'"
+                                    ",member='ActiveChanged'",
+                                    NULL);
+        }
 
         return acquired;
 }
